import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)


/**
 * Write a description of class Board here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Board  extends World
{


    /**
     * Constructor for objects of class Board.
     * 
     */
    public Board()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(800, 600, 1);
          // The buttons need to be on top.
        Class[] paintOrder = new Class[2];
        paintOrder[0] = Button.class;
        paintOrder[1] = Piece.class;
        this.setPaintOrder(paintOrder);
        
        this.addObject(new Raster(), 350,300);
        
        addButtons();
        myGame = new Game();
        myGame.newGame();
        myAI = new AI(myGame);
    }
    
    private boolean firstGame = true;
    
    
    // The game object of the board.
    private Game myGame; 
    
    private AI myAI;
    
    /**
     * Initialises a new Game.
     */
    public void newGame(){
        removeObjects(getObjects(Actor.class));
        addButtons();
        this.addObject(new Raster(), 350,300);
        firstGame = false;
        myGame.newGame();
        myAI = new AI(myGame);
        nextTurn();
    }
    
    /**
     * Add the buttons on the Board.
     */
    private void addButtons(){
        /*
         * Add label AI Player1
         */
        addObject(new Button(this, new GreenfootImage("ButtonAI-P1.png")){
            public void action(){}
        }, 750,25);
        /*
         * Add the Button to toggle the AI of Player 1.
         */
        addObject(new Button(this,new GreenfootImage("ButtonOFF.png")){
            private boolean isON = false;
            {
                if (aiPlayer1 == true)
                    setImage(new GreenfootImage("ButtonON.png"));
            }
            public void action(){
                if (isON == false){
                    setImage(new GreenfootImage("ButtonON.png"));
                    if (firstGame == false){
                        if (getCurrentPiece() != null)
                            removeObject(getCurrentPiece());
                            setCurrentPiece(null);
                            getBoard().toggleAIPlayer1();
                            nextTurn();
                        }
                        else
                            getBoard().toggleAIPlayer1();
                    isON = true;
                }
                else{
                    setImage(new GreenfootImage("ButtonOFF.png"));
                    getBoard().toggleAIPlayer1();
                    isON = false;
                } 
                
            }
        }, 750, 100);
         /*
         * Add label AI Player2
         */
        addObject(new Button(this, new GreenfootImage("ButtonAI-P2.png")){
            public void action(){}
        }, 750,175);
        /*
         * Add the Button to toggle the AI of Player 2.
         */
        addObject(new Button(this,new GreenfootImage("ButtonOFF.png")){
             private boolean isON = false;
            {
                if (aiPlayer2 == true)
                    setImage(new GreenfootImage("ButtonON.png"));
            }
             
             
             public void action(){
                if (isON == false){
                    setImage(new GreenfootImage("ButtonON.png"));
                    if (firstGame == false){
                        if (getCurrentPiece() != null)
                            removeObject(getCurrentPiece());
                            setCurrentPiece(null);
                            getBoard().toggleAIPlayer2();
                            nextTurn();
                    }
                    else
                        getBoard().toggleAIPlayer2();
                    isON = true;
                }
                else{
                    setImage(new GreenfootImage("ButtonOFF.png"));
                     getBoard().toggleAIPlayer2();
                    isON = false;
                } 
               
            }
        }, 750, 250);
         /*
         * Add label New Game.
         */
        addObject(new Button(this, new GreenfootImage("ButtonNEWGAME.png")){
            public void action(){}
        }, 750,325);
        /*
         * Add the Button to start a new Game.
         */
         addObject(new Button(this,new GreenfootImage("ButtonGO.png")){
             public void action(){
                getBoard().newGame();
            }
        }, 750, 400);
        /*
         * Add label Hint. 
         */
         addObject(new Button(this, new GreenfootImage("LabelHINT.png")){
            public void action(){}
        }, 750,475);
         /*
         * Add the Button to get a hint.
         */
         addObject(new Button(this,new GreenfootImage("ButtonHINT.png")){
             public void action(){
                
                int nextMove = myAI.nextMove();
                addObject(new HintPiece(myGame.getTurn(), this.getBoard(), nextMove), Board.getCoordColumn(nextMove), 0);
            }
        }, 750, 550);
        
    }
    
    /**
     * This method returns the x coordinate of a column number. 
     */
    public static int getCoordColumn(int columnnumber){
        return 50 + (100 * (columnnumber - 1));
    }
    
    /**
     * Get the coordinate in y direction of a given column.
     */
    public int getYCoordColumn(int columnNumber){
        return this.getHeight() - (50 + (100 * (myGame.getAmountOfPiecesInColumn(columnNumber))));
    }
    
    /**
     * Do the next move with the given columnnumber
     */
    public void doMove(int columnnumber){
        myGame.doMove(columnnumber);
    }
    
    /**
     * Lets the game know the player is ready for the next turn.
     */
    public void nextTurn(){
        setCurrentPiece(null);
        if ((myGame.checkWin() == Player.NOTHING) && (!myGame.checkDraw())){
            if (! checkAIEnabled(myGame.getTurn())){
                MouseInfo tmpMouse = Greenfoot.getMouseInfo();
                Piece tmpPiece = new Piece(myGame.getTurn(), this);
                setCurrentPiece(tmpPiece);
                if (tmpMouse != null)
                    addObject(tmpPiece, tmpMouse.getX(), tmpMouse.getY());
                else
                    addObject(tmpPiece, 50, 50);
            }
            else{
                    Piece aiPiece = new Piece(myGame.getTurn(), this);
                    int nextMove = myAI.nextMove();
                    aiPiece.doMoveWithPiece(nextMove);
                    addObject(aiPiece, Board.getCoordColumn(nextMove), 0);
            }
            
        }
        else
            displayWin();
    }
    
    // This variable holds the adres of the piece currently controlled by the mouse.
    private Piece currentPiece;
    
    /**
     * This method returns the Piece currently controlled by the player.
     * If their is no Piece, the method returns null.
     */
    public Piece getCurrentPiece(){
        return currentPiece;
    }
    
    /**
     * This method changes the Piece controlled by the mouse.
     */
    public void setCurrentPiece(Piece newPiece){
        currentPiece = newPiece;
    }
    
    /**
     * This method discribes what needs to be done if the game is won. 
     */
    private void displayWin(){
        String winImage = "Player1-WIN.png";
        if (myGame.checkWin() == Player.PLAYER1)
            winImage = "Player1-WIN.png";
        else if (myGame.checkWin() == Player.PLAYER2)
            winImage = "Player2-WIN.png";
        else
            winImage = "Draw.png";
        addObject(new Button(this,new GreenfootImage(winImage)){
             public void action(){
                getBoard().newGame();
                getBoard().removeObject(this);
            }
        }, 350, 300);
    }
    
    /**
     * This method checks if the AI is enabled for a given player.
     * @return  if theis method is invoked with Player.NOTHING the method wil return false.
     */
    private boolean checkAIEnabled(Player playerToCheck){
        if (playerToCheck == Player.PLAYER1)
           return aiPlayer1;
        else if (playerToCheck == Player.PLAYER2)
            return aiPlayer2;
        else // this can only happen if the method is invoke with Player.NOTHING
            return false;
    }
    
    /**
     * Toggle the ai of player 1.
     */
    public void toggleAIPlayer1(){
        if (aiPlayer1 == false)
            aiPlayer1 = true;
        else
            aiPlayer1 = false;
    }
    
    /**
     * Toggle the AI of player 2.
     */
    public void toggleAIPlayer2(){
        if (aiPlayer2 == false)
            aiPlayer2 = true;
        else
            aiPlayer2 = false;
    }
    
    private boolean aiPlayer1 = false;
    private boolean aiPlayer2 = false;
    
        
    
}
