import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A button is an object that preforms an action when is it clicked.
 * 
 * @author Jannes Van Ussel 
 * @version 1.0
 */
public abstract class Button  extends Actor
{
    // Object holding the board the Button is on.
    private Board myBoard;
    
    /**
     * @param myBoard
     *          The board the button is on.
     * @param myImage
     *          The Image of the button to display on the board.
     */
    public Button(Board myBoard, GreenfootImage myImage){
        super();
        this.myBoard = myBoard;
        setImage(myImage);
    }

    /**
     * Act - do whatever the Button wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
       if (Greenfoot.mouseClicked(this))
           action();
    }
    
    /**
     * The action the button must preform.
     */
    public abstract void action();
    
    
    /**
     * Returns the board of the button.
     */
    public Board getBoard(){
        return myBoard;
    }
    
    
}
