public class Game {
    
    
    public Game(){
        newGame();
    }
    
    public void newGame(){
        // First the field must be initialized, every element of the multidimensional array must be initialized/reset.
        initialiseField();
        
        // PLAYER1 has the starting move.
        turn = Player.PLAYER1;
    }
    
    /**
     * Gives the current field.
     * @return
     *          The current field.
     */
    public Player[][] getField(){
        return field;
    }
    
    /**
     * The field of the game.
     */
    private Player[][] field = new Player[6][7];
    
    /**
     * This method initializes a new field, it sets all the spaces in the multidimensional array to Player.NOTHING. 
     */
    public void initialiseField(){
        
        /*
         * This loop runs over all the rows of the multidimensional array.
         */
        for (Player[] pionArray : field){
            
            /*
             *  this loop runs over all the columns of the multidimensional array.
             */
            for (int i = 0; i < pionArray.length; i++){
                /*
                 *  Each element is initialized.
                 */
                pionArray[i] = Player.NOTHING;
            }
        }
    }
    
    /**
     * Do the given move.
     * @param columnNumber
     *          The number of the column to move.
     *              The columnnumber is a number from 1 to 7. 
     * @throws IllegalStateException
     *          An IllegalStateException is thrown if the given move can't be done. 
     */
    public void doMove(int columnNumber) throws IllegalStateException{
        // Invoke the static method do move.
        Game.doMove(columnNumber, field, turn);
        
        // After the move is done, the turn is passed to the other player.
        if (turn.equals(Player.PLAYER1))
            turn = Player.PLAYER2;
        else if (turn.equals(Player.PLAYER2))
            turn = Player.PLAYER1;
    }
    
    /**
     * Do a move on the given Field.
     * @param coulumNumber
     *          Number of the column to move.
     *              The columnnumber is a number from 1 to 7.
     * @param field
     *          The field to apply the move.
     * @param turn
     *          The turn of the player.
     * @return
     *          The field, after the move.
     * @throws IllegalStateException
     *          An IllegalStateException is thrown if the move can not be made.
     */
    public static Player[][] doMove(int coulumNumber, Player[][] field, Player turn) throws IllegalStateException{
        /*
         * Check if the columnnumber is in range.
         */
        if (checkMove(coulumNumber, field)) {
            /*
             * The piece must be at the bottom of the column. To achieve this
             * the loop checks all the spaces in the column, until it finds a
             * free space.
             */
            int i = 0;
            while ((i < field.length)
                    && (field[i][coulumNumber - 1] == Player.NOTHING)) {
                i++;
            }
            if ((i - 1) >= 0) {
                field[i - 1][coulumNumber - 1] = turn;
            } else { // Wrong move!!!
                throw new IllegalStateException();
            }
        }
        return field;
    }
    
    /**
     * Holds a variable to check who's turn it is.
     */
    private Player turn = Player.PLAYER1;
    
    /**
     * Check the given field for a win.
     * @param field
     *          The field to check.
     * @return  The winning player.
     *          If there is no win in the field, this method will return Player.NOTHING as winner.
     */
    public static Player checkWin(Player[][] field){
        // Variables
        Player result;
        Player[] tmpPionArray;
        int fieldHeight = field.length;
        int fieldWidth = field[0].length;
        
        // CHECK ROWS
        for (Player[] pionArray : field){
            result = Game.checkFourInARow(pionArray);
            if (! result.equals(Player.NOTHING)){
                return result;
            }
        }
        
        // CHECK COLUMNS
        for (int i = 0; i < fieldWidth; i++){
            tmpPionArray = new Player[fieldHeight];
            for (int j = 0; j < fieldHeight; j++){
                tmpPionArray[j] = field[j][i]; 
            }
            result = Game.checkFourInARow(tmpPionArray);
            if (! result.equals(Player.NOTHING)){
                return result;
            }
        }
        
        // the upper left corner
        for (int i = 3; i < fieldHeight; i++){
            tmpPionArray = new Player[i + 1];
            for (int j = i; j >= 0; j--)
                tmpPionArray[i - j] = field[j][i - j];
            result = Game.checkFourInARow(tmpPionArray);
            if (! result.equals(Player.NOTHING)){
                return result;
            }
        }
        
        // the lower left corner
        for(int i = 0; i < fieldHeight - 3; i++){
            tmpPionArray = new Player[fieldHeight - i];
            for (int j = 0; j < fieldHeight - i; j++)
                tmpPionArray[j] = field[i + j][j];
            result = Game.checkFourInARow(tmpPionArray);
            if (! result.equals(Player.NOTHING)){
                return result;
            }
        }
        
        // the lower right corner
        // CHECKED
        for (int i = 0; i < fieldHeight - 3; i++){
            tmpPionArray = new Player[fieldHeight - i];
            for (int j = 0; j < fieldHeight - i; j++)
                tmpPionArray[j] = field[i + j][fieldWidth - j - 1];
            result = Game.checkFourInARow(tmpPionArray);
            if (! result.equals(Player.NOTHING)){
                return result;
            }
        }
        
        // the right upper corner
        // CHECKED
        for (int i = 3; i < fieldHeight; i++){
            tmpPionArray = new Player[i + 1];
            for (int j = 0; j < i + 1; j++)
                tmpPionArray[j] = field[i - j][fieldWidth - j - 1];
            result = Game.checkFourInARow(tmpPionArray);
            if (! result.equals(Player.NOTHING)){
                return result;
            }
        }
        return Player.NOTHING;
    }
    
    /**
     * This method checks if there's a four in a row in a given array.
     * @param arrayToCheck
     *          The array to check for a four in a row.
     * @return
     *          The Player who has a four in a row.
     *          If there is no four in a row in the array, this method will return Player.NOTHING.
     */
    private static Player checkFourInARow(Player[] arrayToCheck){
        int counterPLAYER1 = 0;
        int counterPLAYER2 = 0;
        for (Player pion : arrayToCheck){
            if (pion.equals(Player.PLAYER1)){
                counterPLAYER1++;
                if (counterPLAYER2 < 4)
                    counterPLAYER2 = 0;
            }
            else if (pion.equals(Player.PLAYER2)){
                if (counterPLAYER1 < 4)
                    counterPLAYER1 = 0;
                counterPLAYER2++;
            }
            else{
                if (counterPLAYER1 < 4)
                    counterPLAYER1 = 0;
                if (counterPLAYER2 < 4)
                    counterPLAYER2 = 0;
            }
        }
        if (counterPLAYER1 >= 4){
            return Player.PLAYER1;
        }
        else if (counterPLAYER2 >= 4){
            return Player.PLAYER2;
        }
        else
            return Player.NOTHING;
    }
    
    /**
     * Checks if the game if won.
     * If their isn't a four in a row the method returns Player.NOTHING.
     */
    public Player checkWin(){
        return Game.checkWin(getField());
    }
    
    /**
     * Gets the current turn.
     */
    public Player getTurn(){
        return turn;
    }
    
    
    /**
     * This method checks if a given move can be made.
     * @param column
     *          The column to check.
     * @return  True if the move is able to be made.
     */
    public static boolean checkMove (int column, Player[][] fieldToCheck){
        if ((column < 1) || (column > fieldToCheck[1].length + 1))
            return false;
        else
            return fieldToCheck[0][column - 1] == Player.NOTHING;
    }
    
    /**
     * Returns the amount of columns of the field.
     */
    public int getAmountOfColumns(){
        return field[1].length;
    }
    
    /**
     * This method clones a given field.
     * 
     * @param fieldToClone
     *            The field to clone.
     * @return A clone of the given field.
     */
    public static Player[][] cloneField(Player[][] fieldToClone) {
        // Create a temporary variable to store the new field.
        Player[][] tmpField = new Player[fieldToClone.length][fieldToClone[1].length];
        /*
         * The two loop run over the elements of the given field. Each element
         * of the given array is copied to the temporary field.
         */
        for (int i = 0; i < fieldToClone.length; i++) {
            for (int j = 0; j < fieldToClone[1].length; j++) {
                tmpField[i][j] = fieldToClone[i][j];
            }
        }
        /*
         * Each element of the temporary field is the same as the given field,
         * the new field can be returned.
         */
        return tmpField;
    }
    
    /**
     * This method return the amount of pieces in a given column.
     */
    public int getAmountOfPiecesInColumn(int column){
        int counter = 0;
        /*
         * This loop starts from the bottom of the column to check,
         * it keeps running until it reaches an empty space.
         */
        while ((((field.length - 1) - counter) >= 0) &&
                    (field[(field.length - 1) - counter][column - 1] != Player.NOTHING))
            counter++;
        return counter;
    }
    
    /**
     * Check a given array for a three in a row and an empty space.
     * @return  The amount of three in a rows in the array.
     */
    public static int checkThreeInARow(Player playerToCheck, Player[] arrayToCheck){
        int totalCounter = 0;
        
        int playerCounter = 0;
        int nothingCounter = 0;
        for (int i = 0; i < arrayToCheck.length; i++){
            if (arrayToCheck[i] == Player.NOTHING)
                nothingCounter = 1;
            else if (arrayToCheck[i] == playerToCheck)
                playerCounter++;
            else if (arrayToCheck[i] == Player.getOpposite(playerToCheck)){
                nothingCounter = 0;
                playerCounter = 0;
            }
            if ((playerCounter == 3) && (nothingCounter == 1)){
                nothingCounter = 0;
                playerCounter = 0;
                totalCounter++;
            }
            }
            
            return totalCounter;
    }
    
     /**
     * Check a given array for a two in a row and two empty spaces.
     * @return  The amount of three in a rows in the array.
     */
    public static int checkTwoInARow(Player playerToCheck, Player[] arrayToCheck){
        int totalCounter = 0;
        
        int playerCounter = 0;
        int nothingCounter = 0;
        for (int i = 0; i < arrayToCheck.length; i++){
            if (arrayToCheck[i] == Player.NOTHING)
                nothingCounter++;
            else if (arrayToCheck[i] == playerToCheck)
                playerCounter++;
            else if (arrayToCheck[i] == Player.getOpposite(playerToCheck)){
                nothingCounter = 0;
                playerCounter = 0;
            }
            if ((playerCounter == 2) && (nothingCounter > 2)){
                nothingCounter = 0;
                playerCounter = 0;
                totalCounter++;
            }
            }
            
            return totalCounter;
    }
    
    /**
     * Check a given array for a one in a row and three empty spaces.
     * @return  The amount of three in a rows in the array.
     */
    public static int checkOneInARow(Player playerToCheck, Player[] arrayToCheck){
        int totalCounter = 0;
        
        int playerCounter = 0;
        int nothingCounter = 0;
        for (int i = 0; i < arrayToCheck.length; i++){
            if (arrayToCheck[i] == Player.NOTHING)
                nothingCounter++;
            else if (arrayToCheck[i] == playerToCheck)
                playerCounter++;
            else if (arrayToCheck[i] == Player.getOpposite(playerToCheck)){
                nothingCounter = 0;
                playerCounter = 0;
            }
            if ((playerCounter == 1) && (nothingCounter > 3)){
                nothingCounter = 0;
                playerCounter = 0;
                totalCounter++;
            }
            }
            
            return totalCounter;
    }
    
    
    /**
     * This method checks howmany three, two and one in a rows their are in the field. 
     * @return an array with the following numbers:
     *           return[0] == amount of three in a rows PLAYER1
     *           return[1] == amount of two in a rows PLAYER1
     *           return[2] == amount of one in a rows PLAYER1
     *           return[3] == amount of three in a rows PLAYER2
     *           return[4] == amount of two in a rows PLAYER2
     *           return[5] == amount of one in a rows PLAYER2
     */
    public static int[] getAmountThreeTwoAndOneInARow(Player[][] field){
          // Variables
        Player[] tmpPionArray;
        int[] totalCounter = new int[6];
        int fieldHeight = field.length;
        int fieldWidth = field[0].length;
        
        // CHECK ROWS
        for (Player[] pionArray : field){
            totalCounter[0] = totalCounter[0] + Game.checkThreeInARow(Player.PLAYER1, pionArray);
            totalCounter[1] = totalCounter[1] + Game.checkTwoInARow(Player.PLAYER1, pionArray);
            totalCounter[2] = totalCounter[2] + Game.checkOneInARow(Player.PLAYER1, pionArray);
            totalCounter[3] = totalCounter[3] + Game.checkThreeInARow(Player.PLAYER2, pionArray);
            totalCounter[4] = totalCounter[4] + Game.checkTwoInARow(Player.PLAYER2, pionArray);
            totalCounter[5] = totalCounter[5] + Game.checkOneInARow(Player.PLAYER2, pionArray);
        }
        
        // CHECK COLUMNS
        for (int i = 0; i < fieldHeight; i++){
            tmpPionArray = new Player[fieldHeight];
            for (int j = 0; j < fieldHeight; j++)
                tmpPionArray[j] = field[j][i];
                
            totalCounter[0] = totalCounter[0] + Game.checkThreeInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[1] = totalCounter[1] + Game.checkTwoInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[2] = totalCounter[2] + Game.checkOneInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[3] = totalCounter[3] + Game.checkThreeInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[4] = totalCounter[4] + Game.checkTwoInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[5] = totalCounter[5] + Game.checkOneInARow(Player.PLAYER2, tmpPionArray);
        }
        
        // CHECK UPPER DIAGONALS
        for (int i = 3; i < fieldHeight; i++){
            tmpPionArray = new Player[i + 1];
            for (int j = i; j >= 0; j--)
                tmpPionArray[i - j] = field[j][i - j];
                
          totalCounter[0] = totalCounter[0] + Game.checkThreeInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[1] = totalCounter[1] + Game.checkTwoInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[2] = totalCounter[2] + Game.checkOneInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[3] = totalCounter[3] + Game.checkThreeInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[4] = totalCounter[4] + Game.checkTwoInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[5] = totalCounter[5] + Game.checkOneInARow(Player.PLAYER2, tmpPionArray);
        }
        
        for(int i = 0; i < fieldHeight - 3; i++){
            tmpPionArray = new Player[fieldHeight - i];
            for (int j = 0; j < fieldHeight - i; j++)
                tmpPionArray[j] = field[i + j][j];
                
            totalCounter[0] = totalCounter[0] + Game.checkThreeInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[1] = totalCounter[1] + Game.checkTwoInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[2] = totalCounter[2] + Game.checkOneInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[3] = totalCounter[3] + Game.checkThreeInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[4] = totalCounter[4] + Game.checkTwoInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[5] = totalCounter[5] + Game.checkOneInARow(Player.PLAYER2, tmpPionArray);
        }
        
        for (int i = 0; i < fieldHeight - 3; i++){
            tmpPionArray = new Player[fieldHeight - i];
            for (int j = 0; j < fieldHeight - i; j++)
                tmpPionArray[j] = field[i + j][fieldWidth - j - 1];
            
            totalCounter[0] = totalCounter[0] + Game.checkThreeInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[1] = totalCounter[1] + Game.checkTwoInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[2] = totalCounter[2] + Game.checkOneInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[3] = totalCounter[3] + Game.checkThreeInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[4] = totalCounter[4] + Game.checkTwoInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[5] = totalCounter[5] + Game.checkOneInARow(Player.PLAYER2, tmpPionArray);
        }
        
        for (int i = 3; i < fieldHeight; i++){
            tmpPionArray = new Player[i];
            for (int j = 0; j < i; j++)
                tmpPionArray[j] = field[i - j][fieldWidth - j - 1];
            
            totalCounter[0] = totalCounter[0] + Game.checkThreeInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[1] = totalCounter[1] + Game.checkTwoInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[2] = totalCounter[2] + Game.checkOneInARow(Player.PLAYER1, tmpPionArray);
            totalCounter[3] = totalCounter[3] + Game.checkThreeInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[4] = totalCounter[4] + Game.checkTwoInARow(Player.PLAYER2, tmpPionArray);
            totalCounter[5] = totalCounter[5] + Game.checkOneInARow(Player.PLAYER2, tmpPionArray);
        }
        
        return totalCounter;
    }
    
    /**
     * Get the total value of the field.
     * This value is calculated with the amount of three in a row,
     *          the amount of two in a rows
     *          and the amount of one in a rows.
     */
    public static int getValueField(Player[][] field){
       int[] amounts = Game.getAmountThreeTwoAndOneInARow(field);
        
        return (((amounts[0] * 32) + (amounts[1] * 4) + (amounts[2]))
                        - ((amounts[3] * 32) + (amounts[4] * 4) + (amounts[5])));
    }
    
    /**
     * Checks if the game is a draw.
     */
    public boolean checkDraw(){
        int counter = 0;
        for (int i = 1; i < 8; i++){
            if (Game.checkMove(i, this.getField()) == false)
                counter++;
        }
        return counter == 7;
    }
        
}
