import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import javax.swing.Timer;
import javax.swing.AbstractAction;
import javax.swing.Action;
import java.awt.event.ActionEvent;

/**
 * This class represents a Piece on the Board.
 * 
 * @author Jannes Van Ussel 
 * @version 1.0
 */
public class Piece  extends Actor
{

    private Player player = Player.NOTHING;
  
    private Board myBoard;
    
    // The speed the piece moves in x direction.
    private int xSpeed = 1;
    // The acceleration in x direction.
    private int xAcc = 0;
    // The speed the piece moves in y direction.
    private int ySpeed = 2;
    // The acceleration in y direction.
    private int yAcc = 1;

    Piece(Player newPlayer, Board myBoard){
        super();
        // Store the board the piece is on.
        this.myBoard = myBoard;
        // Assign the piece to a player.
        this.player = newPlayer;
        // Set the right image for the player.        
        this.setImage(newPlayer.getFilename());
    }

    /**
     * Act - do whatever the Piece wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        // If the mouse hasn't been clicked, move the piece with the mouse.
        if ((followMouse == true) && (Greenfoot.getMouseInfo() != null))
                this.setLocation(Greenfoot.getMouseInfo().getX(),Greenfoot.getMouseInfo().getY());
        // If the piece isn't following the mouse, it is going to it's place.
        else if (followMouse == false){
                moveToPlace();
                if ((inPlace >= 2) && (turnPassed == false)){
                    myBoard.nextTurn();
                    turnPassed = true;
                }
                else if ((xCoordToMove == getX()) && (yCoordToMove == getY()))
                        inPlace++;
               
            }
        // If the object is clicked, follow mouse becomes false.
        // The piece will not follow the mouse anymore, it will start going to its place.
        if ((Greenfoot.mouseClicked(this)) && (followMouse == true)){
                // local variable storing the closest columnnumber.
                int columnnumber = Piece.checkClosestColumn(
                            Greenfoot.getMouseInfo().getX());
               doMoveWithPiece(columnnumber);
            }
    }
    
    private boolean followMouse = true;
    private boolean turnPassed = false;
    private int inPlace = 0;
    
    /**
     * This method checks witch column is the closest to a given X coordinate.
     * @return  The number of the closest column.
     */
    public static int checkClosestColumn(int x){
        // The closest column is equal to the coordinate divided by 100 + 1.
        // ex. if X = 524 than its column 6. 524/10 = 5 (we are working with JAVA integers)
        if (x >= 700)
            return 7;
        else
            return (x/100)+1;
    }
    
    /**
     * This method moves the piece to a given place, using the variables: xcoordToMove and yCoordToMove
     * The method will not move the piece in one time, the method needs to be invoked multiple times
     * to get to the target location.
     */
    public void moveToPlace(){
        if ((getX() != xCoordToMove) || (getY() != yCoordToMove)){
            // if the x coord is smaller than the one to move to, move a step closer.
            if (getX() < xCoordToMove){
               // The piece must move exacly to the xCoordToMove.
                if ((getX() + xSpeed) > xCoordToMove)
                    setLocation(getX() + xSpeed, getY());
                else
                    setLocation(xCoordToMove, getY());
                }
            // if the y coord is smaller than the one to move to, move a step closer.
            if (getY() < yCoordToMove){
                // The piece must move exacly to the yCoordToMove.
                if ((getY() + ySpeed) < yCoordToMove)
                    setLocation(getX(), getY() + ySpeed);
                else
                    setLocation(getX(), yCoordToMove);
                }
            // if the x coord is larger than the one to move to, move a step closer.
            if (getX() > xCoordToMove){
                 // The piece must move exacly to the xCoordToMove.
                if ((getX() - xSpeed) > xCoordToMove)
                    setLocation(getX() - xSpeed, getY());
                else
                    setLocation(xCoordToMove, getY());
                }
            // if the y coord is larger than the one to move to, move a step closer.
            if (getY() > yCoordToMove){
                // The piece must move exacly to the yCoordToMove.
                if ((getY() - ySpeed) > yCoordToMove)
                    setLocation(getX(), getY() - ySpeed);
                else
                    setLocation(getX(), yCoordToMove);
                }
            
            // Add the acceleration to the xSpeed and ySpeed.
            xSpeed = xSpeed + xAcc;
            ySpeed = ySpeed + yAcc;
        }
        
    }
    
    /**
     * This method sets the x coord the piece will have to move to.
     */
    private void setxCoordToMove(int coord){
        xCoordToMove = coord;
    }
    
    /**
     * This method sets the y coord the piece will have to move to.
     */
    private void setyCoordToMove(int coord){
        yCoordToMove = coord;
    }
    
    private int xCoordToMove;
    private int yCoordToMove;
    
    /**
     * Return the x coordinate the piece is moving to.
     */
    public int getxCoordToMove(){
        return xCoordToMove;
    }
    
    /**
     * Return the y coordinate the piece is moving to.
     */
    public int getyCoordToMove(){
        return yCoordToMove;
    }
    
    /**
     * This method lets the piece stop following the mouse.
     */
    private void disableFollowMouse(){
        followMouse = false;
    }
    
    /**
     * This method lets the piece move to a given column. 
     */
    protected void letPieceMoveTo(int columnnumber){
            setxCoordToMove(myBoard.getCoordColumn(columnnumber));
            setyCoordToMove(myBoard.getYCoordColumn(columnnumber));
            disableFollowMouse();
    }
    
    /**
     * This method does a move with this piece.
     *      It lets the piece move to the given columnnumber,
     *          lets the board know witch move is made.
     */
    public void doMoveWithPiece(int columnnumber){
        letPieceMoveTo(columnnumber);
        myBoard.doMove(columnnumber);
    }
    
    /**
     * Returns the Board of the Piece.
     */
    public Board getBoard(){
        return myBoard;
    }
}
