public class Tree {

    private Player[][] field;
    /** Value of the tree, the value can be 1, 0 or -1 **/
    private int value;
    
    
    
    private int steps;
    
    private Tree[] branche = new Tree[7];
    
    
    public Tree (int steps, Player[][] field, Player turn, Tree parent){
        this.field = field;
        this.turn = turn;
        this.steps = steps;
        myParent = parent;
        
        if (steps > 0){
        
            if (Game.checkWin(field) == Player.PLAYER1)
                this.setValue(Integer.MAX_VALUE);
            else if (Game.checkWin(field) == Player.PLAYER2)
                this.setValue(Integer.MIN_VALUE);
            else{
                this.createBranches();
                this.calcValue();
            }
        }
        else{
            if (Game.checkWin(field) == Player.PLAYER1)
                this.setValue(Integer.MAX_VALUE);
            else if (Game.checkWin(field) == Player.PLAYER2)
                this.setValue(Integer.MIN_VALUE);
            else
                setValue(Game.getValueField(field));
        }
        
            
    }
    
    private Player turn;
    
    /**
     * Returns the turn of the tree.
     */
    private Player getTurn(){
        return turn;
    }
    
    private Tree myParent;
    
    /**
     * Returns the parent of the Tree. 
     */
    private Tree getParent(){
        return myParent;
    }
    
    /**
     * This method calculates the value of the tree by the branches of the tree.
     */
    public void calcValue(){
        
            // if it's PLAYER1's turn we need to find the highest value.
            if (turn == Player.PLAYER1){
                int highestValue = Integer.MIN_VALUE;
                Tree tmpTree;
                for (int i = 1; i <= 7; i++){
                    tmpTree = this.getBrancheByNumber(i);
                    if (tmpTree != null){
                        if (tmpTree.getValue() > highestValue)
                            highestValue = tmpTree.getValue();
                    }
                }
                this.setValue(highestValue);
            }
            // if it's PLAYER2's turn, we need to find the lowest value.
            else if (turn == Player.PLAYER2){
                int lowestValue = Integer.MAX_VALUE;
                Tree tmpTree;
                for (int i = 1; i <= 7; i++){
                    tmpTree = this.getBrancheByNumber(i);
                    if (tmpTree != null){
                        if (tmpTree.getValue() < lowestValue)
                            lowestValue = tmpTree.getValue();
                    }
                }
                this.setValue(lowestValue);
            }
        
            
    }
    
    /**
     * Set the value of the tree
     * @param newValue
     *          The new value of the tree
     * @result  new.getValue = newValue;
     */
    private void setValue(int newValue){
        value = newValue;
    }
    
    /**
     * @return The value of the tree.
     */
    public int getValue(){
        return value;
    }
    
    private void createBranches(){
        if (Game.checkWin(field) == Player.NOTHING){
            for (int i = 1; i <= 7; i++){
                if (Game.checkMove(i, field)){
                   branche[i - 1] = new Tree(steps - 1, Game.doMove(i,Game.cloneField(field), turn), Player.getOpposite(turn), this);
                   
                   
                   /*
                    * Implementation of AB-pruning
                    */
                   Tree tmpParent = getParent();
                   if (tmpParent != null){
                        if ((getTurn() == Player.PLAYER1) && (branche[i - 1].getValue() > tmpParent.getLowestValueBranches()))
                            return;
                        else if ((getTurn() == Player.PLAYER2) && (branche[i - 1].getValue() < tmpParent.getHighestValueBranches()))
                            return;
                        }
                   
                }
            }
        }   
    }
    
    private Tree getBrancheByNumber(int number){
       return branche[number - 1];
    
    }
    
    /**
     * This method gives all the branches of the tree.
     * @return All the branches of the tree is an array of length 7.
     */
    public Tree[] getAllBranches(){
        return branche;
    }
    
    /**
     * This method returns the highest value of the branches.
     */
    public int getHighestValueBranches(){
        int highestValue = Integer.MIN_VALUE;
        for (Tree branche : getAllBranches()){
            if (branche != null){
                int tmpValue = branche.getValue();
                if (tmpValue > highestValue)
                    highestValue = tmpValue;
            }
        }
        return highestValue;
    }
    
    /**
     * This method returns the lowest value of the branches.
     */
    public int getLowestValueBranches(){
        int lowestValue = Integer.MAX_VALUE;
        for (Tree branche : getAllBranches()){
            if (branche != null){
                int tmpValue = branche.getValue();
                if (tmpValue < lowestValue)
                    lowestValue = tmpValue;
            }
        }
        return lowestValue;
    }
}
